package com.ruoyi.web.controller.hotel;

import java.util.List;
import java.util.Map;

import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.hotel.domain.SvipIccards;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.hotel.domain.Sellbills;
import com.ruoyi.hotel.service.ISellbillsService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 消费记录Controller
 *
 * @author ruoyi
 * @date 2021-02-23
 */
@RestController
@RequestMapping("/svip/sellbills")
public class SellbillsController extends BaseController {
    @Autowired
    private ISellbillsService sellbillsService;

    /**
     * 查询消费记录列表
     */
    @RequestMapping("/page")
    public TableDataInfo list(Sellbills sellbills) {
        startPage();
        System.out.println(sellbills.toString());
        List<Map<String, Object>> list = sellbillsService.selectSellMapList(sellbills);
        return getDataTable(list);
    }

    /**
     * 卡作废
     */
    @PreAuthorize("@ss.hasPermi('svip:svipItems:edit')")
    @RequestMapping("/svipCardInvalid")
    @RepeatSubmit
    public AjaxResult svipCardInvalid(@RequestBody SvipIccards svipIccardsDto) {
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return sellbillsService.svipCardInvalid(svipIccardsDto, loginUser.getUser());
    }

    /**
     * 换卡
     */
    @PreAuthorize("@ss.hasPermi('svip:svipItems:edit')")
    @RequestMapping("/svipChange")
    @RepeatSubmit
    public AjaxResult svipChange(@RequestBody SvipIccards svipIccardsDto) {
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return sellbillsService.svipChange(svipIccardsDto, loginUser.getUser());
    }

    /**
     * 导出消费记录列表
     */
    @PreAuthorize("@ss.hasPermi('svip:sellbills:export')")
    @Log(title = "消费记录", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    @RepeatSubmit
    public AjaxResult export(Sellbills sellbills) {
        List<Sellbills> list = sellbillsService.selectSellbillsList(sellbills);
        ExcelUtil<Sellbills> util = new ExcelUtil<Sellbills>(Sellbills.class);
        return util.exportExcel(list, "sellbills");
    }

    /**
     * 获取消费记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('svip:sellbills:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(sellbillsService.selectSellbillsById(id));
    }

    /**
     * 新增消费记录
     */
    @PreAuthorize("@ss.hasPermi('svip:sellbills:add')")
    @Log(title = "消费记录", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    @RepeatSubmit
    public AjaxResult add(@RequestBody Sellbills sellbills) {
        return toAjax(sellbillsService.insertSellbills(sellbills));
    }

    /**
     * 修改消费记录
     */
    @PreAuthorize("@ss.hasPermi('svip:sellbills:edit')")
    @Log(title = "消费记录", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/update")
    @RepeatSubmit
    public AjaxResult edit(@RequestBody Sellbills sellbills) {
        return toAjax(sellbillsService.updateSellbills(sellbills));
    }

    /**
     * 删除消费记录
     */
    @PreAuthorize("@ss.hasPermi('svip:sellbills:remove')")
    @Log(title = "消费记录", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(sellbillsService.deleteSellbillsByIds(ids));
    }
}
