package com.ruoyi.web.controller.hotel;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.system.domain.SysConfig;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.hotel.domain.SvipIccards;
import com.ruoyi.hotel.service.ISvipIccardsService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 会员卡信息Controller
 *
 * @author ruoyi
 * @date 2021-02-23
 */
@RestController
@RequestMapping("/svip/svipIccards")
public class SvipIccardsController extends BaseController {
    @Autowired
    private ISvipIccardsService svipIccardsService;

    /**
     * 查询会员卡信息列表
     */
    @GetMapping("/list")
    public TableDataInfo list(SvipIccards svipIccards) {
        startPage();
        List<SvipIccards> list = svipIccardsService.selectSvipIccardsList(svipIccards);
        return getDataTable(list);
    }

    /**
     * 查询会员卡信息列表
     */
    @GetMapping("/fuzzyQuerySvipIccards")
    public TableDataInfo fuzzyQuerySvipIccards(SvipIccards svipIccards) {
        startPage();
        List<SvipIccards> list = svipIccardsService.selectSvipIccardsList(svipIccards);
        return getDataTable(list);
    }

    /**
     * 修改会员卡警戒线
     */
    @RequestMapping("/selectionPeriod")
    @RepeatSubmit
    public AjaxResult selectionPeriod(@RequestBody SysConfig sysConfigDto) {
        return svipIccardsService.selectionPeriod(sysConfigDto);
    }

    /**
     * 发卡
     */
    @RequestMapping("/openCard")
    @RepeatSubmit
    public AjaxResult openCard(@RequestBody SvipIccards svipIccardsDto) {
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return svipIccardsService.openCard(svipIccardsDto, loginUser.getUser());
    }

    /**
     * 会员卡延期
     */
    @RequestMapping("/svipExtension")
    @RepeatSubmit
    public AjaxResult svipExtension(@RequestBody SvipIccards svipIccardsDto) {
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return svipIccardsService.svipExtension(svipIccardsDto, loginUser.getUser());
    }

    /**
     * 查询 会员卡 状态
     */
    @RequestMapping("/querySvipStatus")
    public AjaxResult querySvipStatus(@RequestBody SvipIccards svipIccardsDto) {
        return svipIccardsService.querySvipStatus(svipIccardsDto);
    }

    /**
     * 查询 会员卡信息
     */
    @RequestMapping("/querySvipIccardsVip")
    public AjaxResult queryVip(SvipIccards svipIccardsDto) {
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return svipIccardsService.querySvipIccardsVip(svipIccardsDto, loginUser.getUser());
    }

    /**
     * 导出会员卡信息列表
     */
    @PreAuthorize("@ss.hasPermi('svip:svipIccards:export')")
    @Log(title = "会员卡信息", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    @RepeatSubmit
    public AjaxResult export(SvipIccards svipIccards) {
        List<SvipIccards> list = svipIccardsService.selectSvipIccardsList(svipIccards);
        ExcelUtil<SvipIccards> util = new ExcelUtil<SvipIccards>(SvipIccards.class);
        return util.exportExcel(list, "svipIccards");
    }

    /**
     * 获取会员卡信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('svip:svipIccards:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(svipIccardsService.selectSvipIccardsById(id));
    }

    /**
     * 新增会员卡信息
     */
    @PreAuthorize("@ss.hasPermi('svip:svipIccards:add')")
    @Log(title = "会员卡信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    @RepeatSubmit
    public AjaxResult add(@RequestBody SvipIccards svipIccards) {
        if (StringUtils.isNull(svipIccards.getCardId())) {
            return AjaxResult.error("操作失败，会员卡信息有误");
        }
        if (StringUtils.isEmpty(svipIccards.getCardNo()) || "".equals(svipIccards.getCardNo())) {
            return AjaxResult.error("操作失败，会员卡信息有误");
        }
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return svipIccardsService.insertSvipIccards(svipIccards, loginUser.getUser());
    }

    @RequestMapping("/user")
    public Map<String, Object> user() {
        Map<String, Object> map = new HashMap<>();
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        map.put("time", df.format(new Date()));
        map.put("sysUser", loginUser.getUser());
        map.put("user", loginUser.getUser().getNickName());
        return map;
    }

    /**
     * 修改会员卡信息
     */
    @PreAuthorize("@ss.hasPermi('svip:svipIccards:edit')")
    @Log(title = "会员卡信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/update")
    @RepeatSubmit
    public AjaxResult edit(@RequestBody SvipIccards svipIccards) {
        return toAjax(svipIccardsService.updateSvipIccards(svipIccards));
    }

    /**
     * 删除会员卡信息
     */
    @PreAuthorize("@ss.hasPermi('svip:svipIccards:remove')")
    @Log(title = "会员卡信息", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(svipIccardsService.deleteSvipIccardsByIds(ids));
    }
}
