package com.ruoyi.web.controller.hotel;

import java.util.List;
import java.util.Map;

import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.hotel.domain.SvipIccards;
import com.ruoyi.hotel.service.ISellbillsService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.hotel.domain.SvipItems;
import com.ruoyi.hotel.service.ISvipItemsService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 会员卡类型Controller
 *
 * @author ruoyi
 * @date 2021-02-23
 */
@RestController
@RequestMapping("/svip/svipItems")
public class SvipItemsController extends BaseController {
    @Autowired
    private ISvipItemsService svipItemsService;

    @Autowired
    private ISellbillsService sellbillsService;

    /**
     * 查询会员卡类型列表
     */
    @GetMapping("/list")
    public TableDataInfo list(SvipItems svipItems) {
        startPage();
        List<SvipItems> list = svipItemsService.selectSvipItemsMapList(svipItems);
        return getDataTable(list);
    }

    /**
     * 导出会员卡类型列表
     */
    @PreAuthorize("@ss.hasPermi('svip:svipItems:export')")
    @Log(title = "会员卡类型", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(SvipItems svipItems) {
        List<SvipItems> list = svipItemsService.selectSvipItemsList(svipItems);
        ExcelUtil<SvipItems> util = new ExcelUtil<SvipItems>(SvipItems.class);
        return util.exportExcel(list, "svipItems");
    }

    /**
     * 获取会员卡类型详细信息
     */
    @PreAuthorize("@ss.hasPermi('svip:svipItems:query')")
    @GetMapping(value = "/{itemId}")
    public AjaxResult getInfo(@PathVariable("itemId") Long itemId) {
        return AjaxResult.success(svipItemsService.selectSvipItemsById(itemId));
    }

    /**
     * 新增会员卡类型
     */
//    @PreAuthorize("@ss.hasPermi('svip:svipItems:add')")
    @Log(title = "会员卡类型", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResult add(@RequestBody SvipItems svipItems) {
        return toAjax(svipItemsService.insertSvipItems(svipItems));
    }

    /**
     * 消费保存
     */
    @RequestMapping("/bookkeepingSales")
    public AjaxResult saveSales(
            @RequestParam("sellbills") String tdSalesDto,
            @RequestParam("iccards") String svipIccardsDto) {
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return svipItemsService.bookkeepingSales(tdSalesDto, svipIccardsDto, loginUser.getUser());
    }

    /**
     * 修改会员卡类型
     */
    @PreAuthorize("@ss.hasPermi('svip:svipItems:edit')")
    @Log(title = "会员卡类型", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/update")
    public AjaxResult edit(@RequestBody SvipItems svipItems) {
        return toAjax(svipItemsService.updateSvipItems(svipItems));
    }

    /**
     * 删除会员卡类型
     */
    @PreAuthorize("@ss.hasPermi('svip:svipItems:remove')")
    @Log(title = "会员卡类型", businessType = BusinessType.DELETE)
    @DeleteMapping("/{itemIds}")
    public AjaxResult remove(@PathVariable Long[] itemIds) {
        return toAjax(svipItemsService.deleteSvipItemsByIds(itemIds));
    }
}
