package com.ruoyi.hotel.service.impl;

import java.util.*;

import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.hotel.domain.SvipIccards;
import com.ruoyi.hotel.mapper.SvipIccardsMapper;
import com.ruoyi.system.mapper.SysDeptMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.hotel.mapper.SellbillsMapper;
import com.ruoyi.hotel.domain.Sellbills;
import com.ruoyi.hotel.service.ISellbillsService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

/**
 * 消费记录Service业务层处理
 *
 * @author ruoyi
 * @date 2021-02-23
 */
@Service
public class SellbillsServiceImpl implements ISellbillsService {
    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private SysDeptMapper sysDeptMapper;

    @Autowired
    private SellbillsMapper sellbillsMapper;

    @Autowired
    private SvipIccardsMapper svipIccardsMapper;

    /**
     * 查询消费记录
     *
     * @param id 消费记录ID
     * @return 消费记录
     */
    @Override
    public Sellbills selectSellbillsById(Long id) {
        return sellbillsMapper.selectSellbillsById(id);
    }

    /**
     * 查询消费记录列表
     *
     * @param sellbills 消费记录
     * @return 消费记录
     */
    @Override
    public List<Sellbills> selectSellbillsList(Sellbills sellbills) {
        return sellbillsMapper.selectSellbillsList(sellbills);
    }

    /**
     * 查询消费记录列表
     *
     * @param sellbills 消费记录
     * @return 消费记录
     */
    @Override
    public List<Map<String, Object>> selectSellMapList(Sellbills sellbills) {
        return sellbillsMapper.selectSellbillsMapList(sellbills);
    }

    /**
     * 新增消费记录
     *
     * @param sellbills 消费记录
     * @return 结果
     */
    @Override
    @Transactional
    public int insertSellbills(Sellbills sellbills) {
        return sellbillsMapper.insertSellbills(sellbills);
    }

    /**
     * 修改消费记录
     *
     * @param sellbills 消费记录
     * @return 结果
     */
    @Override
    @Transactional
    public int updateSellbills(Sellbills sellbills) {
        return sellbillsMapper.updateSellbills(sellbills);
    }

    /**
     * 批量删除消费记录
     *
     * @param ids 需要删除的消费记录ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteSellbillsByIds(Long[] ids) {
        return sellbillsMapper.deleteSellbillsByIds(ids);
    }

    /**
     * 删除消费记录信息
     *
     * @param id 消费记录ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteSellbillsById(Long id) {
        return sellbillsMapper.deleteSellbillsById(id);
    }

    /**
     * 卡作废
     *
     * @return
     */
    @Override
    @Transactional
    public AjaxResult svipCardInvalid(SvipIccards svipIccards, SysUser userDto) {
        SvipIccards iccards = svipIccardsMapper.selectSvipIccardsById(svipIccards.getId());
        if (StringUtils.isNull(iccards)) {
            return AjaxResult.error("未找到会员卡信息");
        }
        if ("S".equals(iccards.getStatus())) {
            return AjaxResult.error("操作失败，会员卡已停用");
        }
        SysUser user = sysUserMapper.selectUserById(iccards.getGrantId());
        if (StringUtils.isNull(user)) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("未找到用户信息");
        }
        //获取当前年月
        Calendar cal = Calendar.getInstance();
        long year = cal.get(Calendar.YEAR);
        int month = cal.get(Calendar.MONTH) + 1;
        Sellbills sellbills = new Sellbills();
        sellbills.setBillNo("单号");
        sellbills.setPbillNo("主单号");
        sellbills.setAccYear(year);
        sellbills.setAccMonth(month);
        sellbills.setCreateDate(new Date());
        sellbills.setBillDate(new Date());
        sellbills.setApplyDate(new Date());
        sellbills.setAuditDate(new Date());
        sellbills.setPostDate(new Date());
        sellbills.setUserBane(userDto.getUserName());
        sellbills.setActId(410L);
        sellbills.setBillStatus(6L);
        sellbills.setDeptId(userDto.getDeptId());
        sellbills.setPrintTime(new Date());
        sellbills.setCardId(iccards.getId());
        sellbills.setCardNo(iccards.getCardNo());
        sellbills.setItemId(iccards.getItemId());
        sellbills.setItemType(iccards.getItemType());
        sellbills.setVipDr(iccards.getVipDr());
        sellbills.setVipBlc(iccards.getVipBlc());
        sellbills.setCorpName(iccards.getCorpName());
        sellbills.setCorpTel(iccards.getCorpTel());
        sellbills.setCorpAddr(iccards.getCorpAddr());
        sellbills.setCorpidCardno(iccards.getCorpIdcardNo());
        sellbills.setAllNotes(svipIccards.getRemarks());
        sellbills.setCreateBy(userDto.getUserName());
        sellbills.setCreateDate(new Date());
        int sellInsert = sellbillsMapper.insertSellbills(sellbills);
        svipIccards.setStatus("S");
        svipIccards.setUpdateTime(new Date());
        svipIccards.setRemarks(iccards.getRemarks());
        svipIccards.setUpdateBy(userDto.getUserName());
        int iccardUpdate = svipIccardsMapper.updateSvipIccards(svipIccards);
        if (sellInsert <= 0 && iccardUpdate <= 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("修改失败,更新数据失败");
        }
        return AjaxResult.success();
    }

    /**
     * 换卡
     */
    @Override
    @Transactional
    public AjaxResult svipChange(SvipIccards svipIccardsDto, SysUser userDto) {
        SvipIccards newIccard = new SvipIccards();
        newIccard.setCardId(svipIccardsDto.getCardId());
        newIccard.setCardNo(svipIccardsDto.getCardNo());
        List<SvipIccards> newIccardList = svipIccardsMapper.selectSvipIccardsList(newIccard);
        if (Objects.equals(newIccardList.size(), 0) || newIccardList.size() > 1) {
            return AjaxResult.error("会员卡信息有误,请确认");
        }
        SvipIccards svipIccards = newIccardList.get(0);
        if ("A".equals(svipIccards.getStatus())) {
            return AjaxResult.error("该会员卡已开卡,无法确认");
        } else if ("S".equals(svipIccards.getStatus())) {
            return AjaxResult.error("该会员卡已停用,无法确认");
        }
        SvipIccards iccards = svipIccardsMapper.selectSvipIccardsById(svipIccardsDto.getId());
        svipIccards.setItemId(iccards.getItemId());
        svipIccards.setItemType(iccards.getItemType());
        svipIccards.setVipDr(iccards.getVipDr());
        svipIccards.setVipCr(iccards.getVipCr());
        svipIccards.setVipBlc(iccards.getVipBlc());
        svipIccards.setStatus(iccards.getStatus());
        svipIccards.setInitDate(iccards.getInitDate());
        svipIccards.setInitOpname(iccards.getInitOpname());
        svipIccards.setCorpName(iccards.getCorpName());
        svipIccards.setCorpTel(iccards.getCorpTel());
        svipIccards.setCorpAddr(iccards.getCorpAddr());
        svipIccards.setCorpIdcardNo(iccards.getCorpIdcardNo());
        svipIccards.setValIdFrom(iccards.getValIdFrom());
        svipIccards.setValIdTo(iccards.getValIdTo());
        svipIccards.setGrantDate(iccards.getGrantDate());
        svipIccards.setGrantOpname(iccards.getGrantOpname());
        svipIccards.setRemarks(iccards.getRemarks());
        svipIccards.setUpdateBy(userDto.getUserName());
        svipIccards.setUpdateTime(new Date());
        int newIccardsUpdat = svipIccardsMapper.updateSvipIccards(svipIccards);
        iccards.setStatus("S");
        int iccardsUpdate = svipIccardsMapper.updateSvipIccards(iccards);
        //获取当前年月
        Calendar cal = Calendar.getInstance();
        long year = cal.get(Calendar.YEAR);
        int month = cal.get(Calendar.MONTH) + 1;
        Sellbills sellbills = new Sellbills();
        sellbills.setBillNo("单号");
        sellbills.setPbillNo("主单号");
        sellbills.setAccYear(year);
        sellbills.setAccMonth(month);
        sellbills.setCreateDate(new Date());
        sellbills.setBillDate(new Date());
        sellbills.setApplyDate(new Date());
        sellbills.setAuditDate(new Date());
        sellbills.setPostDate(new Date());
        sellbills.setUserBane(userDto.getUserName());
        sellbills.setActId(210L);
        sellbills.setBillStatus(6L);
        sellbills.setDeptId(userDto.getDeptId());
        sellbills.setPrintTime(new Date());
        sellbills.setCardId(iccards.getId());
        sellbills.setCardNo(iccards.getCardNo());
        sellbills.setItemId(iccards.getItemId());
        sellbills.setItemType(iccards.getItemType());
        sellbills.setVipDr(iccards.getVipDr());
        sellbills.setVipBlc(iccards.getVipBlc());
        sellbills.setCorpName(iccards.getCorpName());
        sellbills.setCorpTel(iccards.getCorpTel());
        sellbills.setCorpAddr(iccards.getCorpAddr());
        sellbills.setCorpidCardno(iccards.getCorpIdcardNo());
        sellbills.setAllNotes(svipIccardsDto.getRemarks());
        int sellInsert = sellbillsMapper.insertSellbills(sellbills);
        if (iccardsUpdate <= 0 && newIccardsUpdat <= 0 && sellInsert <= 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("会员卡更换失败");
        }
        return AjaxResult.success();
    }
}
