package com.ruoyi.hotel.service.impl;

import java.text.SimpleDateFormat;
import java.util.*;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.hotel.domain.Sellbills;
import com.ruoyi.hotel.domain.SvipItems;
import com.ruoyi.hotel.mapper.SellbillsMapper;
import com.ruoyi.hotel.mapper.SvipItemsMapper;
import com.ruoyi.system.domain.SysConfig;
import com.ruoyi.system.mapper.SysConfigMapper;
import com.ruoyi.system.mapper.SysDeptMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.hotel.mapper.SvipIccardsMapper;
import com.ruoyi.hotel.domain.SvipIccards;
import com.ruoyi.hotel.service.ISvipIccardsService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

/**
 * 会员卡信息Service业务层处理
 *
 * @author ruoyi
 * @date 2021-02-23
 */
@Service
public class SvipIccardsServiceImpl implements ISvipIccardsService {
    @Autowired
    private SvipIccardsMapper svipIccardsMapper;

    @Autowired
    private SvipItemsMapper svipItemsMapper;

    @Autowired
    private SysConfigMapper sysConfigMapper;

    @Autowired
    private SellbillsMapper sellbillsMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private SysDeptMapper sysDeptMapper;

    @Autowired
    private RedisCache redisCache;

    /**
     * 查询会员卡信息
     *
     * @param id 会员卡信息ID
     * @return 会员卡信息
     */
    @Override
    public SvipIccards selectSvipIccardsById(Long id) {
        return svipIccardsMapper.selectSvipIccardsById(id);
    }

    /**
     * 查询会员卡信息列表
     *
     * @param svipIccards 会员卡信息
     * @return 会员卡信息
     */
    @Override
    public List<SvipIccards> selectSvipIccardsList(SvipIccards svipIccards) {
        return svipIccardsMapper.selectSvipIccardsList(svipIccards);
    }

    /**
     * 新增会员卡信息
     *
     * @param svipIccards 会员卡信息
     * @param user
     * @return 结果
     */
    @Override
    @Transactional
    public AjaxResult insertSvipIccards(SvipIccards svipIccards, SysUser user) {
        SvipIccards iccard = new SvipIccards();
        iccard.setCardNo(svipIccards.getCardNo());
        if (svipIccardsMapper.selectSvipIccardsList(iccard).size() > 0) {
            return AjaxResult.error("会员卡号已被使用");
        }
        SvipIccards iccardId = new SvipIccards();
        iccardId.setCardId(svipIccards.getCardId());
        if (svipIccardsMapper.selectSvipIccardsList(iccardId).size() > 0) {
            return AjaxResult.error("会员卡芯片号已被使用");
        }
        SvipItems svipItems = svipItemsMapper.selectSvipItemsById(svipIccards.getItemId());
        svipIccards.setVipDr(svipItems.getAmount());
        svipIccards.setVipBlc(svipItems.getAmount());
        svipIccards.setStatus("N");
        svipIccards.setGrantId(user.getUserId());
        svipIccards.setInitDate(new Date());
        svipIccards.setCreateBy(user.getUserName());
        svipIccards.setInitOpname(user.getNickName());
        if (svipIccardsMapper.insertSvipIccards(svipIccards) <= 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("操作失败，新增会员卡失败，请联系管理员");
        }
        return AjaxResult.success();
    }

    /**
     * 修改会员卡信息
     *
     * @param svipIccards 会员卡信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateSvipIccards(SvipIccards svipIccards) {
        return svipIccardsMapper.updateSvipIccards(svipIccards);
    }

    /**
     * 批量删除会员卡信息
     *
     * @param ids 需要删除的会员卡信息ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteSvipIccardsByIds(Long[] ids) {
        return svipIccardsMapper.deleteSvipIccardsByIds(ids);
    }

    /**
     * 删除会员卡信息信息
     *
     * @param id 会员卡信息ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteSvipIccardsById(Long id) {
        return svipIccardsMapper.deleteSvipIccardsById(id);
    }

    @Override
    @Transactional
    public AjaxResult selectionPeriod(SysConfig sysConfigDto) {
        SysConfig config = new SysConfig();
        config.setConfigKey("alert_time");
        List<SysConfig> sysConfigs = sysConfigMapper.selectConfigList(config);
        if (Objects.equals(sysConfigs.size(), 0)) {
            return AjaxResult.error("未找到会员卡警戒设置实际参数");
        }
        redisCache.setCacheObject(getCacheKey("alert_time"), sysConfigDto.getConfigValue());
        SysConfig sysConfig = sysConfigs.get(0);
        sysConfig.setConfigValue(sysConfigDto.getConfigValue());
        int configUpdate = sysConfigMapper.updateConfig(sysConfig);
        if (configUpdate <= 0) {
            return AjaxResult.error("修改失败，请重新操作");
        }
        return AjaxResult.success();
    }

    /**
     * 设置cache key
     *
     * @param configKey 参数键
     * @return 缓存键key
     */
    private String getCacheKey(String configKey)
    {
        return Constants.SYS_CONFIG_KEY + configKey;
    }

    /**
     * 发卡
     */
    @Override
    @Transactional
    public AjaxResult openCard(SvipIccards svipIccardsDto, SysUser userDto) {
        SvipIccards iccards = svipIccardsMapper.selectSvipIccardsById(svipIccardsDto.getId());
        if (StringUtils.isNull(iccards)) {
            return AjaxResult.error("操作失败，未找到会员卡信息");
        }
        if ("A".equals(iccards.getStatus())) {
            return AjaxResult.error("操作失败，该会员卡已被激活");
        }
        if ("S".equals(iccards.getStatus())) {
            return AjaxResult.error("操作失败，该会员卡已被停用");
        }
        svipIccardsDto.setStatus("A");
        svipIccardsDto.setGrantDate(new Date());
        svipIccardsDto.setGrantOpname(userDto.getUserName());
        svipIccardsDto.setVipBlc(svipIccardsDto.getVipDr());
        //获取当前年月
        Calendar cal = Calendar.getInstance();
        long year = cal.get(Calendar.YEAR);
        int month = cal.get(Calendar.MONTH) + 1;
        Sellbills sellbills = new Sellbills();
        sellbills.setBillNo("单号");
        sellbills.setPbillNo("主单号");
        sellbills.setAccYear(year);
        sellbills.setAccMonth(month);
        sellbills.setCreateDate(new Date());
        sellbills.setBillDate(new Date());
        sellbills.setApplyDate(new Date());
        sellbills.setAuditDate(new Date());
        sellbills.setPostDate(new Date());
        sellbills.setUserBane(userDto.getUserName());
        sellbills.setActId(101L);
        sellbills.setBillStatus(6L);
        sellbills.setDeptId(userDto.getDeptId());
        sellbills.setPrintTime(new Date());
        sellbills.setCardId(svipIccardsDto.getId());
        sellbills.setCardNo(svipIccardsDto.getCardNo());
        sellbills.setItemId(svipIccardsDto.getItemId());
        sellbills.setItemType(svipIccardsDto.getItemType());
        sellbills.setVipDr(svipIccardsDto.getVipDr());
        sellbills.setVipBlc(svipIccardsDto.getVipBlc());
        sellbills.setCorpName(svipIccardsDto.getCorpName());
        sellbills.setCorpTel(svipIccardsDto.getCorpTel());
        sellbills.setCorpAddr(svipIccardsDto.getCorpAddr());
        sellbills.setCorpidCardno(svipIccardsDto.getCorpIdcardNo());
        sellbills.setAllNotes(svipIccardsDto.getRemarks());
        int sellinstar = sellbillsMapper.insertSellbills(sellbills);
        svipIccardsDto.setRemarks(iccards.getRemarks());
        int iccardUpdate = svipIccardsMapper.updateSvipIccards(svipIccardsDto);
        if (sellinstar <= 0 || iccardUpdate <= 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("修改失败 请重新操作");
        }
        return AjaxResult.success();
    }

    /**
     * 会员卡延期
     */
    @Override
    @Transactional
    public AjaxResult svipExtension(SvipIccards svipIccards, SysUser user) {
        SvipIccards iccards = svipIccardsMapper.selectSvipIccardsById(svipIccards.getId());
        if (StringUtils.isNull(iccards)) {
            return AjaxResult.error("未找到会员卡信息");
        }
        if ("S".equals(iccards.getStatus())) {
            return AjaxResult.error("操作失败，该会员卡已被停用");
        }
        //获取当前年月
        Calendar cal = Calendar.getInstance();
        long year = cal.get(Calendar.YEAR);
        int month = cal.get(Calendar.MONTH) + 1;
        Sellbills sellbills = new Sellbills();
        sellbills.setBillNo("单号");
        sellbills.setPbillNo("主单号");
        sellbills.setAccYear(year);
        sellbills.setAccMonth(month);
        sellbills.setCreateDate(new Date());
        sellbills.setBillDate(new Date());
        sellbills.setApplyDate(new Date());
        sellbills.setAuditDate(new Date());
        sellbills.setPostDate(new Date());
        sellbills.setUserBane(user.getUserName());
        sellbills.setActId(310L);
        sellbills.setBillStatus(6L);
        sellbills.setDeptId(user.getDeptId());
        sellbills.setPrintTime(new Date());
        sellbills.setCardId(iccards.getId());
        sellbills.setCardNo(iccards.getCardNo());
        sellbills.setItemId(iccards.getItemId());
        sellbills.setItemType(iccards.getItemType());
        sellbills.setVipDr(iccards.getVipDr());
        sellbills.setVipBlc(iccards.getVipBlc());
        sellbills.setCorpName(iccards.getCorpName());
        sellbills.setCorpTel(iccards.getCorpTel());
        sellbills.setCorpAddr(iccards.getCorpAddr());
        sellbills.setCorpidCardno(iccards.getCorpIdcardNo());
        sellbills.setAllNotes(svipIccards.getRemarks());
        int insertSellbills = sellbillsMapper.insertSellbills(sellbills);
        svipIccards.setRemarks(iccards.getRemarks());
        int updateSvipIccards = svipIccardsMapper.updateSvipIccards(svipIccards);
        if (insertSellbills <= 0 || updateSvipIccards <= 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("修改失败，请重新编辑提交");
        }
        return AjaxResult.success();
    }

    /**
     * 查询 会员卡 状态
     */
    @Override
    @Transactional
    public AjaxResult querySvipStatus(SvipIccards svipIccardsDto) {
        List<SvipIccards> svipIccards = svipIccardsMapper.selectSvipIccardsList(svipIccardsDto);
        if (Objects.equals(svipIccards.size(), 0) || svipIccards.size() > 1) {
            return AjaxResult.error("会员卡信息有误, 请确认");
        }
        SvipIccards iccards = svipIccards.get(0);
        switch (iccards.getStatus()) {
            case "N":
                return AjaxResult.error("会员卡未开卡");
            case "S":
                return AjaxResult.error("会员卡已被停用");
            default:
        }
        SimpleDateFormat df = new SimpleDateFormat("HH:mm");//设置日期格式
        boolean flag = DateUtils.belongCalendar(new Date(), iccards.getValIdFrom(), iccards.getValIdTo());
        if (!flag) {
            return AjaxResult.error("会员卡已过期");
        }
        if (iccards.getVipBlc() < 0) {
            if ("T".equals(iccards.getItemType())) {
                return AjaxResult.error("会员卡余额不足");
            } else {
                return AjaxResult.error("会员卡次数不足");
            }
        }
        return AjaxResult.success();
    }

    /**
     * 查询 会员卡信息
     */
    @Override
    @Transactional
    public AjaxResult querySvipIccardsVip(SvipIccards svipIccardsDto, SysUser user) {
        Map<String, Object> map = new HashMap<>();
        List<SvipIccards> iccards = svipIccardsMapper.selectSvipIccardsList(svipIccardsDto);
        if (Objects.equals(iccards.size(), 0) || iccards.size() > 1) {
            return AjaxResult.error("会员卡信息有误，请确认");
        }
        SvipIccards svipIccards = iccards.get(0);
        map.put("svipIccards", svipIccards);
        SvipItems svipItems = svipItemsMapper.selectSvipItemsById(svipIccards.getItemId());
        if (StringUtils.isNull(svipItems)) {
            return AjaxResult.error("未找到会员卡类型");
        }
        map.put("svipItems", svipItems);
        Date date = new Date();
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(Calendar.YEAR, svipItems.getExpirationDate().intValue());
        cal.getTime();
        map.put("newTime", new Date());
        map.put("valIdTo", cal.getTime());
        map.put("userName", user.getNickName());
        SysUser sysUser = new SysUser();
        sysUser.setUserId(svipIccards.getGrantId());
        SysDept sysDept = sysDeptMapper.selectDeptByUser(sysUser);
        map.put("dept", sysDept);
        return AjaxResult.success(map);
    }
}
