package com.ruoyi.hotel.service.impl;

import java.util.*;

import com.alibaba.fastjson.JSONArray;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.hotel.domain.Sellbills;
import com.ruoyi.hotel.domain.SvipIccards;
import com.ruoyi.hotel.mapper.SellbillsMapper;
import com.ruoyi.hotel.mapper.SvipIccardsMapper;
import com.ruoyi.system.mapper.SysDeptMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.hotel.mapper.SvipItemsMapper;
import com.ruoyi.hotel.domain.SvipItems;
import com.ruoyi.hotel.service.ISvipItemsService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

/**
 * 会员卡类型Service业务层处理
 *
 * @author ruoyi
 * @date 2021-02-23
 */
@Service
public class SvipItemsServiceImpl implements ISvipItemsService {
    @Autowired
    private SvipIccardsMapper svipIccardsMapper;

    @Autowired
    private SvipItemsMapper svipItemsMapper;

    @Autowired
    private SellbillsMapper sellbillsMapper;

    @Autowired
    private SysDeptMapper sysDeptMapper;

    /**
     * 查询会员卡类型
     *
     * @param itemId 会员卡类型ID
     * @return 会员卡类型
     */
    @Override
    public SvipItems selectSvipItemsById(Long itemId) {
        return svipItemsMapper.selectSvipItemsById(itemId);
    }

    /**
     * 查询会员卡类型列表
     *
     * @param svipItems 会员卡类型
     * @return 会员卡类型
     */
    @Override
    public List<SvipItems> selectSvipItemsList(SvipItems svipItems) {
        return svipItemsMapper.selectSvipItemsList(svipItems);
    }

    /**
     * 新增会员卡类型
     *
     * @param svipItems 会员卡类型
     * @return 结果
     */
    @Override
    public int insertSvipItems(SvipItems svipItems) {
        return svipItemsMapper.insertSvipItems(svipItems);
    }

    /**
     * 修改会员卡类型
     *
     * @param svipItems 会员卡类型
     * @return 结果
     */
    @Override
    public int updateSvipItems(SvipItems svipItems) {
        return svipItemsMapper.updateSvipItems(svipItems);
    }

    /**
     * 批量删除会员卡类型
     *
     * @param itemIds 需要删除的会员卡类型ID
     * @return 结果
     */
    @Override
    public int deleteSvipItemsByIds(Long[] itemIds) {
        return svipItemsMapper.deleteSvipItemsByIds(itemIds);
    }

    /**
     * 删除会员卡类型信息
     *
     * @param itemId 会员卡类型ID
     * @return 结果
     */
    @Override
    public int deleteSvipItemsById(Long itemId) {
        return svipItemsMapper.deleteSvipItemsById(itemId);
    }

    @Override
    public List<SvipItems> selectSvipItemsMapList(SvipItems svipItems) {
        return svipItemsMapper.selectSvipItemsList(svipItems);
    }

    /**
     * 消费保存
     */
    @Override
    @Transactional
    public AjaxResult bookkeepingSales(String tdSalesDto, String svipIccardsDto, SysUser user) {
        //获取当前年月
        Calendar cal = Calendar.getInstance();
        long year = cal.get(Calendar.YEAR);
        int month = cal.get(Calendar.MONTH) + 1;
        SvipIccards svipIccards = JSONArray.parseObject(svipIccardsDto, SvipIccards.class);
        SvipIccards iccard = new SvipIccards();
        iccard.setCardId(svipIccards.getCardId());
        iccard.setCardNo(svipIccards.getCardNo());
        List<SvipIccards> iccardList = svipIccardsMapper.selectSvipIccardsList(iccard);
        if (Objects.equals(iccardList.size(), 0) || iccardList.size() > 1) {
            return AjaxResult.error("会员卡信息有误，请联系管理员");
        }
        SvipIccards iccards = iccardList.get(0);
        if ("S".equals(iccards.getStatus())) {
            return AjaxResult.error("未找到会员卡或会员卡无效");
        }
        if (iccards.getVipBlc() < svipIccards.getVipCr()) {
            return AjaxResult.error("会员卡余额不足");
        }
        svipIccards.setRemarks(iccards.getRemarks());
        svipIccards.setVipCr(svipIccards.getVipCr() + iccards.getVipCr());
        int iccardUpdate = svipIccardsMapper.updateSvipIccards(svipIccards);
        svipIccards.setId(iccards.getId());
        Sellbills sellbills = JSONArray.parseObject(tdSalesDto, Sellbills.class);
        sellbills.setAccYear(year);
        sellbills.setAccMonth(month);
        sellbills.setCreateDate(new Date());
        sellbills.setBillDate(new Date());
        sellbills.setApplyDate(new Date());
        sellbills.setAuditDate(new Date());
        sellbills.setPostDate(new Date());
        sellbills.setUserBane(user.getUserName());
        sellbills.setActId(110L);
        sellbills.setBillStatus(6L);
        sellbills.setDeptId(user.getDeptId());
        sellbills.setCardId(iccards.getId());
        sellbills.setCardNo(iccards.getCardNo());
        sellbills.setItemId(iccards.getItemId());
        sellbills.setItemType(iccards.getItemType());
        sellbills.setVipDr(svipIccards.getVipDr());
        sellbills.setVipBlc(iccards.getVipBlc());
        sellbills.setCorpName(iccards.getCorpName());
        sellbills.setCorpTel(iccards.getCorpTel());
        sellbills.setCorpAddr(iccards.getCorpAddr());
        sellbills.setCorpidCardno(iccards.getCorpIdcardNo());
        sellbills.setRemarks(svipIccards.getRemarks());
        int sellInsert = sellbillsMapper.insertSellbills(sellbills);
        if (iccardUpdate <= 0 || sellInsert <= 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("更新数据失败，请联系管理员");
        }
        return AjaxResult.success();
    }
}
